@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 """
-Copyright (C) 2008-2014 Wolfgang Rohdewald <wolfgang@rohdewald.de>
+Copyright (C) 2008-2016 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
 Kajongg is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -27,11 +27,11 @@ from collections import defaultdict
 from functools import total_ordering
 
 from twisted.internet.defer import succeed
-from util import gitHead
+from util import gitHead, CsvWriter
 from rand import CountingRandom
-from log import logError, logWarning, logException, logDebug, m18n
-from common import Internal, IntDict, Debug, Options, unicodeString, unicode
-from common import isPython3
+from log import logError, logWarning, logException, logDebug, i18n
+from common import Internal, IntDict, Debug, Options
+from common import StrMixin, Speeds
 from wind import Wind, East
 from query import Query
 from rule import Ruleset
@@ -40,11 +40,11 @@ from sound import Voice
 from wall import Wall
 from move import Move
 from player import Players, Player, PlayingPlayer
-from compat import CsvWriter
+from animation import animateAndDo, AnimationSpeed
 
 
 @total_ordering
-class HandId(object):
+class HandId(StrMixin):
 
     """handle a string representing a hand Id"""
 
@@ -103,7 +103,7 @@ class HandId(object):
         self.roundsFinished = handWind.__index__()
         if self.roundsFinished > ruleset.minRounds:
             logWarning(
-                u'Ruleset %s has %d minimum rounds but you want round %d(%s)'
+                'Ruleset %s has %d minimum rounds but you want round %d(%s)'
                 % (ruleset.name, ruleset.minRounds, self.roundsFinished + 1,
                    handWind))
             self.roundsFinished = ruleset.minRounds
@@ -111,16 +111,16 @@ class HandId(object):
         self.rotated = int(handId[1]) - 1
         if self.rotated > 3:
             logWarning(
-                u'You want %d rotations, reducing to maximum of 3' %
+                'You want %d rotations, reducing to maximum of 3' %
                 self.rotated)
             self.rotated = 3
             return
         for char in handId[2:]:
             if char < 'a':
-                logWarning(u'you want %s, changed to a' % char)
+                logWarning('you want %s, changed to a' % char)
                 char = 'a'
             if char > 'z':
-                logWarning(u'you want %s, changed to z' % char)
+                logWarning('you want %s, changed to z' % char)
                 char = 'z'
             self.notRotated = self.notRotated * 26 + ord(char) - ord('a') + 1
 
@@ -175,9 +175,6 @@ class HandId(object):
     def __str__(self):
         return self.prompt()
 
-    def __repr__(self):
-        return 'HandId({})'.format(self.prompt())
-
     def __eq__(self, other):
         return (other
                 and (self.roundsFinished, self.rotated, self.notRotated) ==
@@ -191,7 +188,7 @@ class HandId(object):
             other.roundsFinished, other.rotated, other.notRotated)
 
 
-class Game(object):
+class Game:
 
     """the game without GUI"""
     # pylint: disable=too-many-instance-attributes
@@ -211,7 +208,7 @@ class Game(object):
         assert self.__class__ != Game, 'Do not directly instantiate Game'
         for wind, name in names:
             assert isinstance(wind, Wind), 'Game.__init__ expects Wind objects'
-            assert isinstance(name, (str, unicode)), 'Game.__init__: name must be string and not {}'.format(type(name))
+            assert isinstance(name, str), 'Game.__init__: name must be string and not {}'.format(type(name))
         self.players = Players()
         # if we fail later on in init, at least we can still close the program
         self.myself = None
@@ -225,11 +222,7 @@ class Game(object):
         self.roundsFinished = 0
         self._currentHandId = None
         self._prevHandId = None
-        self.randomGenerator = CountingRandom(self)
         self.wantedGame = wantedGame
-        self._setHandSeed()
-        self.activePlayer = None
-        self.__winner = None
         self.moves = []
         self.gameid = gameid
         self.playOpen = False
@@ -244,6 +237,10 @@ class Game(object):
         # tile names are always lowercase
         self.dangerousTiles = list()
         self.csvTags = []
+        self.randomGenerator = CountingRandom(self)
+        self._setHandSeed()
+        self.activePlayer = None
+        self.__winner = None
         self._setGameId()
         self.__useRuleset(ruleset)
         # shift rules taken from the OEMC 2005 rules
@@ -404,12 +401,16 @@ class Game(object):
         The server tells us the seating order and player names.
 
         @param playerNames: A list of 4 tuples. Each tuple holds wind and name.
-        @type playerNames: The tuple contents must be C{unicode}
+        @type playerNames: The tuple contents must be C{str}
         @todo: Can we pass L{Players} instead of that tuple list?
         """
         if not self.players:
-            self.players = Players(self.playerClass(
-                self, playerNames[x][1]) for x in range(4))
+            self.players = Players()
+            for idx in range(4):
+                # append each separately: Until they have names, the current length of players
+                # is used to assign one of the four walls to the player
+                self.players.append(self.playerClass(
+                    self, playerNames[idx][1]))
         for wind, name in playerNames:
             self.players.byName(name).wind = wind
         if self.client and self.client.name:
@@ -446,6 +447,9 @@ class Game(object):
                     self.players = Players(self.players[1:] + self.players[:1])
                 for idx, player in enumerate(self.players):
                     player.front = self.wall[idx]
+                    player.sideText.board = player.front
+                # we want names to move simultaneously
+                self.players[1].sideText.refreshAll()
 
     @staticmethod
     def _newGameId():
@@ -487,7 +491,7 @@ class Game(object):
         return int(self.wantedGame.split('/')[0])
 
     def _setHandSeed(self):  # TODO: move this to PlayingGame
-        """set seed to a reproducable value, independent of what happend
+        """set seed to a reproducible value, independent of what happened
         in previous hands/rounds.
         This makes it easier to reproduce game situations
         in later hands without having to exactly replay all previous hands"""
@@ -529,13 +533,13 @@ class Game(object):
         """save computed values to database,
         update score table and balance in status line"""
         scoretime = datetime.datetime.now().replace(microsecond=0).isoformat()
-        logMessage = u''
+        logMessage = ''
         for player in self.players:
             if player.hand:
                 manualrules = '||'.join(x.rule.name
                                         for x in player.hand.usedRules)
             else:
-                manualrules = m18n('Score computed manually')
+                manualrules = i18n('Score computed manually')
             Query(
                 "INSERT INTO SCORE "
                 "(game,hand,data,manualrules,player,scoretime,won,prevailing,"
@@ -547,8 +551,8 @@ class Game(object):
                  player.handTotal, player.payment, player.balance,
                  self.rotated, self.notRotated),
                 (player.hand.string, manualrules))
-            logMessage += u'{player:<12} {hand:>4} {total:>5} {won} | '.format(
-                player=unicode(player)[:12], hand=player.handTotal,
+            logMessage += '{player:<12} {hand:>4} {total:>5} {won} | '.format(
+                player=str(player)[:12], hand=player.handTotal,
                 total=player.balance,
                 won='WON' if player == self.winner else '   ')
             for usedRule in player.hand.usedRules:
@@ -565,7 +569,7 @@ class Game(object):
         if result:
             if Debug.explain:
                 if not self.belongsToRobotPlayer():
-                    self.debug(u','.join(x.name for x in result), prevHandId=True)
+                    self.debug(','.join(x.name for x in result), prevHandId=True)
             self.rotateWinds()
         return bool(result)
 
@@ -593,6 +597,9 @@ class Game(object):
             if self.roundsFinished % 4 and self.rotated == 0:
                 # exchange seats between rounds
                 self.__exchangeSeats()
+            if Internal.scene:
+                with AnimationSpeed(Speeds.windMarker):
+                    self.wall.showWindMarkers()
 
     def debug(self, msg, btIndent=None, prevHandId=False):
         """
@@ -607,18 +614,18 @@ class Game(object):
         @type prevHandId: C{bool}
         """
         if self.belongsToRobotPlayer():
-            prefix = u'R'
+            prefix = 'R'
         elif self.belongsToHumanPlayer():
-            prefix = u'C'
+            prefix = 'C'
         elif self.belongsToGameServer():
-            prefix = u'S'
+            prefix = 'S'
         else:
             logDebug(msg, btIndent=btIndent)
             return
         handId = self._prevHandId if prevHandId else self.handId
-        handId = unicodeString(handId.prompt(withMoveCount=True))
+        handId = handId.prompt(withMoveCount=True)
         logDebug(
-            u'%s%s: %s' % (prefix, handId, unicodeString(msg)),
+            '%s%s: %s' % (prefix, handId, msg),
             withGamePrefix=False,
             btIndent=btIndent)
 
@@ -629,7 +636,7 @@ class Game(object):
         try:
             return Players.allNames[playerid]
         except KeyError:
-            return m18n('Player %1 not known', playerid)
+            return i18n('Player %1 not known', playerid)
 
     @classmethod
     def loadFromDB(cls, gameid, client=None):
@@ -662,7 +669,7 @@ class Game(object):
                 tuple([qGameRecord[wind], wind.char, 0, False, East.char])
                 for wind in Wind.all4])
         if len(qScoreRecords) != 4:
-            logError(u'game %d inconsistent: There should be exactly '
+            logError('game %d inconsistent: There should be exactly '
                      '4 score records for the last hand' % gameid)
 
         # after loading SQL, prepare values.
@@ -671,7 +678,7 @@ class Game(object):
         # did not, we get no record here. Should we try to fix this or
         # exclude such a game from the list of resumable games?
         if len(set(x[4] for x in qScoreRecords)) != 1:
-            logError(u'game %d inconsistent: All score records for the same '
+            logError('game %d inconsistent: All score records for the same '
                      'hand must have the same prevailing wind' % gameid)
 
         players = list(tuple([Wind(x[1]), Game.__getName(x[0])])
@@ -687,7 +694,7 @@ class Game(object):
             player = game.players.byId(playerid)
             if not player:
                 logError(
-                    u'game %d inconsistent: player %d missing in game table' %
+                    'game %d inconsistent: player %d missing in game table' %
                     (gameid, playerid))
             else:
                 player.getsPayment(record[2])
@@ -698,7 +705,8 @@ class Game(object):
         game.notRotated += 1
         game.maybeRotateWinds()
         game.sortPlayers()
-        game.wall.decorate()
+        with AnimationSpeed(Speeds.windMarker):
+            animateAndDo(game.wall.decorate4)
         return game
 
     def finished(self):
@@ -802,11 +810,11 @@ class PlayingGame(Game):
             myself.voice = Voice.locate(myself.name)
             if myself.voice:
                 if Debug.sound:
-                    logDebug(u'myself %s gets voice %s' % (
+                    logDebug('myself %s gets voice %s' % (
                         myself.name, myself.voice))
             else:
                 if Debug.sound:
-                    logDebug(u'myself %s gets no voice' % (myself.name))
+                    logDebug('myself %s gets no voice' % (myself.name))
 
     def writeCsv(self):
         """write game summary to Options.csv"""
@@ -819,7 +827,7 @@ class PlayingGame(Game):
             if Options.rounds:
                 self.csvTags.append('ROUNDS:%s' % Options.rounds)
             row = [self.ruleset.name, Options.AI,
-                   gitHead(), '3' if isPython3 else '2',
+                   gitHead(), '3',
                    str(self.seed), ','.join(self.csvTags)]
             for player in sorted(self.players, key=lambda x: x.name):
                 row.append(player.name)
@@ -991,13 +999,13 @@ class PlayingGame(Game):
                 # remote human player sent her voice, or we are human
                 # and have a voice
                 if Debug.sound and player != self.myself:
-                    logDebug(u'%s got voice from opponent: %s' % (
+                    logDebug('%s got voice from opponent: %s' % (
                         player.name, player.voice))
             else:
                 player.voice = Voice.locate(player.name)
                 if player.voice:
                     if Debug.sound:
-                        logDebug(u'%s has own local voice %s' % (
+                        logDebug('%s has own local voice %s' % (
                             player.name, player.voice))
             if player.voice:
                 for voice in Voice.availableVoices():
@@ -1016,7 +1024,7 @@ class PlayingGame(Game):
                 player.voice = predefined.pop(0)
                 if Debug.sound:
                     logDebug(
-                        u'%s gets one of the still available voices %s' % (
+                        '%s gets one of the still available voices %s' % (
                             player.name, player.voice))
 
     def dangerousFor(self, forPlayer, tile):
@@ -1054,10 +1062,10 @@ class PlayingGame(Game):
                         if not x.isBonus]
             for tile in allTiles:
                 assert isinstance(tile, Tile), tile
-            # see http://www.logilab.org/ticket/23986
+            # see https://www.logilab.org/ticket/23986
             invisibleTiles = set(x for x in allTiles
                                  if x not in self.visibleTiles)
-            msg = m18n('Short living wall: Tile is invisible, hence dangerous')
+            msg = i18n('Short living wall: Tile is invisible, hence dangerous')
             self.dangerousTiles = list(
                 x for x in self.dangerousTiles if x[1] != msg)
             self.dangerousTiles.append((invisibleTiles, msg))
